/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using NAnt.Core.Types;
using System;
using NAnt.Core;

namespace TopCoder.Test.Framework.Servers
{
    /// <summary>
    /// <p>
    /// This class is an abstract base class for all the classes representing a server to be used when
    /// running tests. It stores some attributes passed from the ServerElement class.
    /// The appropriate properties are provided for accessing the attributes.
    /// It also has a properties for accessing the nested
    /// "sourcefiles" and "credentials" elements of the "server" XML element.
    /// The other important thing is a property for
    /// specifing the options of the server.
    /// </p>
    /// <p>
    /// The class provides the abstract methods of starting/stopping/initializing data/clean-up,
    /// which should be implemented by the derived classes.
    /// </p>
    /// <p>
    /// This class is mutable, i.e. not thread-safe.
    /// </p>
    /// </summary>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public abstract class Server
    {

        /// <summary>
        /// This string constanst represents the name (key) of the option, the derived classes
        /// should use for getting the timeout, which is a time in milliseconds, the StartUp() method
        /// should wait for the server to start. If the specified time passes and the server is not running
        /// the StartUp() method should throw an exception.
        /// The behaviour described here is only a suggestion, because the derived classes
        /// can treat options in any way they like to do.
        /// </summary>
        public const string TIMEOUT = "timeout";

        /// <summary>
        /// This string constanst represents the name (key) of the option, the derived classes
        /// should use for getting the number of port,  used to connect the server.
        /// Note, that the derived classes
        /// can treat options in any way they like to do.
        /// </summary>
        public const string PORT = "port";

        /// <summary>
        /// This string constanst represents the name (key) of the option, the derived classes
        /// should use for getting the address of machine running the server.
        /// Note, that the derived classes
        /// can treat options in any way they like to do.
        /// </summary>
        public const string ADDRESS = "address";

        /// <summary>
        /// This integer constanst represents maximum port number.
        /// </summary>
        private const int MAX_PORT_NUMBER = 65535;

        /// <summary>
        /// <p>
        /// This variable represents the "startserver" attribute.
        /// "startserver" attribute specifies if the server should
        /// be started by the Component when running tests.
        /// This attribute is not required, the default value is true.
        /// Initially is set to true.
        /// Can be accessed via the StartServer property.
        /// </p>
        /// </summary>
        private bool startServer = true;

        /// <summary>
        /// <p>
        /// This variable represents the "initserver" attribute.
        /// "initserver" attribute specifies if the server should
        /// be intialized with data by the Component when running tests.
        /// This attribute is not required, the default value is true.
        /// Is initialized in the constructor.
        /// Can be accessed via the InitServer property.
        /// </p>
        /// </summary>
        private bool initServer = true;

        /// <summary>
        /// This variable represents the collection of nested "option" XML elements, which
        /// represent the options of the server.
        /// The options can be any key-value pairs, where keys and values are strings.
        ///
        /// The derived classes may interpret the options in a freevolous way.
        /// The string constants representing the names of the most common
        /// options are provided by the class.
        /// They are : TIMEOUT, PORT and ADDRESS.
        /// The derived classes shoud provide a similar string constants
        /// representing the names of options they need.
        ///
        /// Initially is null.
        /// Can be accessed via the "Options" property.
        /// </summary>
        private OptionCollection options = null;

        /// <summary>
        /// This variable represents the nested "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.
        /// This includes user name, password an domain.
        /// This nested element can possibly be ommited,
        /// it depends on the particular server.
        ///
        /// Can be accessed via the "Credentials" property.
        /// </summary>
        private Credential credentials = null;

        /// <summary>
        /// This variable represents the nested "sourcefiles" XML element, which
        /// represents the files used when initializing the server with data.
        /// This can be an application to deploy for an application server or
        /// some SQL scripts, DB tables, etc. for a data server,  or some other files
        /// the particular server may need to be initialized.
        /// This nested element can possibly be ommited if the
        /// derived from Server class permits it to be ommited.
        ///
        /// Can be accessed via the "SourceFiles" property.
        /// </summary>
        private FileSet sourceFiles = null;

        /// <summary>
        /// The current NAnt Project.
        ///
        /// Can be accessed via the "Project" property.
        /// </summary>
        private Project project = null;

        /// <summary>
        /// This property provides the access to the "startServer" variable.
        /// The "startServer" variable represents "StartServer" attribute.
        /// "startserver" attribute specifies if the server should
        /// be started by the Component when running tests.
        /// This attribute is not required, the default value is true.
        /// This attribute is required.
        /// </summary>
        ///
        /// <value>The value which indicates if the server should
        /// be started by the Component when running tests.</value>
        public bool StartServer
        {
            get
            {
                return startServer;
            }
            set
            {
                startServer = value;
            }
        }

        /// <summary>
        /// This property provides the access to the "initServer" variable.
        /// The "initServer" variable represents "InitServer" attribute.
        /// "initserver" attribute specifies if the server should
        /// be intialized with data by the Component when running tests.
        /// This attribute is not required, the default value is true.
        /// </summary>
        ///
        /// <value>The value which indicates if the server should
        /// be intialized with data by the Component when running tests.</value>
        public bool InitServer
        {
            get
            {
                return initServer;
            }
            set
            {
                initServer = value;
            }
        }

        /// <summary>
        /// This property provides the access to the "options" variable.
        /// The "options" variable represents the collection of nested "option" XML elements, which
        /// represent the options of the server.
        /// The options can be any key-value pairs, where keys and values are strings.
        ///
        /// The string constants representing the names of the most common
        /// options are provided by the class.
        /// They are : TIMEOUT, PORT and ADDRESS.
        /// The derived classes shoud provide a similar string constants
        /// representing the names of options they need.
        /// </summary>
        ///
        /// <value>The collection of nested "option" XML elements, which represent the options
        /// of the server.</value>
        public OptionCollection Options
        {
            get
            {
                return options;
            }
            set
            {
                options = value;
            }
        }


        /// <summary>
        /// This property provides the access to the "credentials" variable.
        /// The "credentials" variable represents the nested "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.
        /// This includes user name, password an domain.
        /// This nested element can possibly be ommited,
        /// it depends on the particular server.
        /// </summary>
        ///
        /// <value>The nested "credentials" XML element, which
        /// represents the credentials used to authenticate to the server.</value>
        public Credential Credentials
        {
            get
            {
                return credentials;
            }
            set
            {
                credentials = value;
            }
        }

        /// <summary>
        /// This property provides the access to the "sourceFiles" variable.
        /// The "sourceFiles" variable represents the nested "sourcefiles" XML element, which
        /// represents the files used when initializing the server with data.
        /// This can be an application to deploy for an application server or
        /// some SQL scripts, DB tables, etc. for a data server,  or some other files
        /// the particular server may need to be initialized.
        /// This nested element can possibly be ommited if the
        /// derived from Server class permits it to be ommited.
        /// </summary>
        ///
        /// <value>The nested "sourcefiles" XML element, which represents the files used when initializing
        /// the server with data.</value>
        public FileSet SourceFiles
        {
            get
            {
                return sourceFiles;
            }
            set
            {
                sourceFiles = value;
            }
        }

        /// <summary>
        /// This property provides the access to the "project" variable.
        /// The "project" variable represents the current NAnt project.
        /// </summary>
        ///
        /// <value>The current NAnt project.</value>
        public Project Project
        {
            get
            {
                return project;
            }
            set
            {
                project = value;
            }
        }

        /// <summary>
        /// This property provides the read access to the option, the derived classes
        /// should use for getting the timeout, which is a time in milliseconds, the StartUp() method
        /// should wait for the server to start. If the specified time passes and the server is not running
        /// the StartUp() method should throw an exception.
        /// The behaviour described here is only a suggestion, because the derived classes
        /// can treat options in any way they like to do.
        /// </summary>
        ///
        /// <value>Timeout which is a time in milliseconds.</value>
        /// <exception cref="ServerException">If timeout option is missing, or the option value is not
        /// a valid integer value, or the value is negative.</exception>
        public int Timeout
        {
            get
            {
                return GetPositiveIntOption(TIMEOUT);
            }
        }

        /// <summary>
        /// This property provides the read access to the option, the derived classes
        /// should use for getting the number of port used to connect the server.
        /// Note that the derived classes can treat options in any way they like to do.
        /// </summary>
        ///
        /// <value>Number of port used to connect the server.</value>
        /// <exception cref="ServerException">If port option is missing, or the option value is not
        /// a valid integer value, or the value is negative, or the port value is larger than 65535.</exception>
        public int Port
        {
            get
            {
                int port =  GetPositiveIntOption(PORT);

                if (port > MAX_PORT_NUMBER)
                {
                    throw new ServerException(PORT + " option should port number value in range of valid port.");
                }

                return port;
            }
        }

        /// <summary>
        /// Gets the parsed integer value of specified option by name.
        /// This private method is used by <c>Port</c> getter property and <c>Timeout</c> getter property.
        /// </summary>
        ///
        /// <param name="optionName">Name of option needed to get.</param>
        /// <returns>The parsed option value.</returns>
        /// <exception cref="ServerException">If the specified option is missing, or the option value is not
        /// a valid integer value, or the value is negative.</exception>
        private int GetPositiveIntOption(string optionName)
        {
            int optionValue = 0;

            // try to parse the string
            try
            {
                optionValue = Int32.Parse(GetOption(optionName));
            }
            catch (FormatException fe)
            {
                throw new ServerException(optionName + " option value should be valid format integer.", fe);
            }
            catch (OverflowException oe)
            {
                throw new ServerException(optionName + " option value should be valid int-type integer.", oe);
            }

            // negative value is illegal
            if (optionValue < 0)
            {
                throw new ServerException(optionName + " option value should be non-negative.");
            }

            return optionValue;
        }

        /// <summary>
        /// This property provides the read access to the option, the derived classes
        /// should use for getting the address of machine running the server.
        /// Note that the derived classes can treat options in any way they like to do.
        /// </summary>
        ///
        /// <value>Address of machine running the server.</value>
        /// <exception cref="ServerException">If Options is null, or the specified option is inexistent.</exception>
        public string Address
        {
            get
            {
                return GetOption(ADDRESS);
            }
        }

        /// <summary>
        /// Gets specified option from <c>Options</c> property by name.
        /// </summary>
        ///
        /// <param name="optionName">Name of option to get.</param>
        /// <returns>Option get from Options.</returns>
        /// <exception cref="ServerException">If Options is null, or the specified option is inexistent.</exception>
        internal string GetOption(string optionName)
        {
            if (Options == null)
            {
                throw new ServerException("Options is null.");
            }

            Option option = Options[optionName];

            if (option == null)
            {
                throw new ServerException(optionName + " option is missing.");
            }
            return option.Value;

        }

        /// <summary>
        /// <p>
        /// Creates an instance of class.
        /// </p>
        /// </summary>
        protected Server()
        {
            // Just do nothing
        }

        /// <summary>
        /// <p>
        /// Starts the server.
        /// In the case when "startserver" attribute is set to false,
        /// the server should not be started, but some initialization
        /// may still be needed.
        /// Subclasses must implement this method to perform
        /// the actual task of starting up the server.
        /// </p>
        /// </summary>
        ///
        /// <exception cref="ServerException">If any error happens.</exception>
        public abstract void StartUp();

        /// <summary>
        /// <p>
        /// Stops the server.
        /// The server should not be stopped if it wasn't started by the startUp() method,
        /// i.e.  "startserver" attribute is set to false.
        /// Subclasses must implement this method to perform
        /// the actual task of stopping the server.
        /// </p>
        /// </summary>
        ///
        /// <exception cref="ServerException">If any error happens.</exception>
        public abstract void ShutDown();

        /// <summary>
        /// <p>
        /// Inits the server  with data.
        /// In the case when "initserver" attribute is set to false,
        /// does nothing, or can check if the data are properly initialized,
        /// the implementation can choose the preffered way.
        /// Subclasses must implement this method to perform
        /// the actual task of initializing server with data.
        /// </p>
        /// </summary>
        ///
        /// <exception cref="ServerException">If any error happens.</exception>
        public abstract void InitData();

        /// <summary>
        /// <p>
        /// Clean-ups the data initialized by the initData() method.
        /// In the case when "initserver" attribute is set to false
        /// or data was not initialized, does nothing.
        /// Subclasses must implement this method to perform
        /// the actual task of  cleaning-up the data.
        /// </p>
        /// </summary>
        ///
        /// <exception cref="ServerException">If any error happens.</exception>
        public abstract void CleanUpData();

    }
}
