/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using System.ServiceProcess;
using System.ComponentModel;
using NAnt.Core.Tasks;
using NAnt.Core.Types;
using NAnt.Core;
using TopCoder.Test.Framework.Servers;

namespace TopCoder.Test.Framework.Servers.Web
{

    /// <summary>
    /// <p>
    /// This class is derived from ApplicationServer class.
    /// It represents web application server to be used while running tests.
    /// It supports the Microsoft IIS server.
    /// Really it should support almost any service-based servers.
    /// </p>
    /// <p>
    /// Supprted options:<br/>
    /// The name of service of the server is specified
    /// via the SERVICE_NAME option.<br/>
    /// The DEPLOYMENT_DIR option from the AppicationServer is used,
    /// it is required when the "initserver" attribute is true.<br/>
    /// The TIMEOUT option from the Server is used,
    /// it is required, it specifies the time to wait for service to reach the needed state.<br/>
    /// The defaut name of service, that correspons to IIS server
    /// is represented by DEFAULT_SERVICE_NAME constant.<br/>
    /// </p>
    /// <p>
    /// The NUnitAsp can be used for writing tests for IIS server
    /// when using this componenent.
    /// </p>
    /// <p>
    /// The parent class is mutable, so this class is mutable, i.e. not thread-safe.
    /// </p>
    /// </summary>
    ///
    /// <author>real_vg</author>
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public class IISServer : ApplicationServer
    {
        /// <summary>
        /// This string constanst represents the name (key) of the option, the used
        /// for getting the name of service of the server.
        /// This option is not required.
        /// The DEFAULT_SERVICE_NAME constant is used as a default value of option.
        /// </summary>
        public const string SERVICE_NAME = "service_name";

        /// <summary>
        /// This string constant represents default value for the SERVICE_NAME option.
        /// It the name of IIS service.
        /// </summary>
        private const string DEFAULT_SERVICE_NAME = "w3svc";

        /// <summary>
        /// This string constant represents default value for the TIMEOUT option.
        /// The default value is a time of 1 minute, represented in milliseconds.
        /// </summary>
        private const int DEFAULT_TIMEOUT = 60000;

        /// <summary>
        /// <p>
        /// Creates an instance of class.
        /// </p>
        /// </summary>
        public IISServer() : base()
        {
            // Just call superclass constructor
        }

        /// <summary>
        /// <p>
        /// Starts the server. This method will also deploy the application to DEPLOYMENT_DIR,
        /// if InitServer property is true. If the server can't be started, ServerException will be thrown,
        /// and the deployed application will be undeployed.
        /// </p>
        /// </summary>
        ///
        /// <exception cref="ServerException">if any error happens.</exception>
        public override void StartUp()
        {
            // At first the application should be deployed if InitServer is true
            if (InitServer) {
                Deploy();
            }

            // start the service, and undeploy the application if any exception is thrown
            try
            {
                StartService();
            }
            catch (ServerException)
            {
                // try to undeploy silently.
                SilentUndeploy();
                throw;
            }

        }

        /// <summary>
        /// <p>
        /// Stops the server, and possibly clean-ups the data.
        /// </p>
        /// <p>
        /// Stopping the server means that the service corresponding to the server is stopped.
        /// If the service doesn't reach the needed state in the time
        /// specified by the TIMEOUT option, an exception is thrown.
        /// After stopping the server the web-application is deleted from the deployment dir.
        /// (If the "initserver" attribute is true)
        /// </p>
        /// </summary>
        ///
        /// <exception cref="ServerException">If any error happens.</exception>
        public override void ShutDown()
        {
            // stop the service first, and undeploy silently if any error occurs.
            try
            {
                StopService();
            }
            catch (ServerException)
            {
                SilentUndeploy();
                throw;
            }

            // undeploy if the application are deployed
            if (InitServer)
            {
                Undeploy();
            }
        }


        /// <summary>
        /// <p>
        /// Inits the server with data.
        /// </p>
        /// <p>
        /// This implementation of the method just does nothing.
        /// (The application is deployed in StartUp() method)
        /// </p>
        /// </summary>
        public override void InitData()
        {
            // do nothing here, The application is deployed in StartUp() method
        }

        /// <summary>
        /// <p>
        /// Clean-ups the data initialized by the initData() method.
        /// </p>
        /// <p>
        /// This implementation of the method just does nothing.
        /// (Application is removed from deployment dir in ShutDown() method)
        /// </p>
        /// </summary>
        public override void CleanUpData()
        {
            // do nothing here, The application is removed from deployment dir in ShutDown() method
        }

        /// <summary>
        /// Gets the service name from options. Return DEFAULT_SERVICE_NAME if it does not exist.
        /// </summary>
        /// <returns>the service name</returns>
        private string GetServiceName()
        {
            if (Options == null)
            {
                return DEFAULT_SERVICE_NAME;
            }
            if (Options[SERVICE_NAME] == null)
            {
                return DEFAULT_SERVICE_NAME;
            }
            string serviceName = Options[SERVICE_NAME].Value;
            if (serviceName == null || serviceName.Trim().Length == 0)
            {
                return DEFAULT_SERVICE_NAME;
            }
            return serviceName;

        }

        /// <summary>
        /// Gets the timeout from options. Return DEFAULT_TIMEOUT if it does not exist.
        /// </summary>
        /// <returns>the timeout option</returns>
        /// <exception cref="ServerException">Timeout option is invalid</exception>
        private int GetTimeout()
        {
            if (Options[TIMEOUT] == null)
            {
                return DEFAULT_TIMEOUT;
            }
            return Timeout;
        }

        /// <summary>
        /// Starts the server. Any errors will be wrapped in ServerException.
        /// </summary>
        ///
        /// <exception cref="ServerException">if any error happens.</exception>
        private void StartService()
        {

            string serviceName = GetServiceName();
            int timeout = GetTimeout();

            try
            {
                ServiceController serviceController = new ServiceController(serviceName);

                // Check if the service is running
                if (serviceController.Status == ServiceControllerStatus.Running)
                {
                    // If it is running, the ServerException should be thrown
                    throw new ServerException("The service has already started.");
                }

                // Start the service
                if (serviceController.Status == ServiceControllerStatus.Paused)
                {
                    serviceController.Continue();
                }
                else if (serviceController.Status == ServiceControllerStatus.Stopped)
                {
                    serviceController.Start();
                }
                else
                {
                    throw new ServerException("The service status of IIS is illegal.");
                }
                // Wait for the service to reach the desired status, or the timeout expired
                serviceController.WaitForStatus(ServiceControllerStatus.Running, TimeSpan.FromMilliseconds(timeout));

            }
            catch (ServerException)
            {
                // do not wrap ServerException
                throw;
            }
            catch (Win32Exception e)
            {
                throw new ServerException("Fails to start the server.", e);
            }
            catch (InvalidOperationException e)
            {
                throw new ServerException("Fails to start the server.", e);
            }
            catch (System.ServiceProcess.TimeoutException e)
            {
                throw new ServerException("Fails to start the server(timeout).", e);
            }
        }

        /// <summary>
        /// <p>
        /// Deploy the source files to server. This method copys all source files to the deploy directory.
        /// </p>
        /// </summary>
        ///
        /// <exception cref="ServerException">if any error happens.</exception>
        private void Deploy()
        {
            // Create a NAnt copy task
            CopyTask copyTask = new CopyTask();
            copyTask.Project = Project;

            // Specify destination directory
            copyTask.ToDirectory = DeploymentDirectory;

            // Specify files to copy
            copyTask.CopyFileSet = SourceFiles;

            // Specify that files should be overwritten
            copyTask.Overwrite = true;

            try
            {
                copyTask.Execute();
            }
            catch (ArgumentException e)
            {
                // ArgumentException may be thrown if the deployment directory or source file set is invalid
                throw new ServerException("Failed to copy the files.", e);
            }
            catch (BuildException e)
            {
                throw new ServerException("Failed to copy the files.", e);
            }

        }

        /// <summary>
        /// Stop the server. Any errors will be wrapped in ServerException.
        /// </summary>
        ///
        /// <exception cref="ServerException">if any error happens.</exception>
        private void StopService()
        {

            string serviceName = GetServiceName();
            int timeout = GetTimeout();

            try
            {
                ServiceController serviceController = new ServiceController(serviceName);

                if (serviceController.CanStop)
                {
                    serviceController.Stop();
                }
                else
                {
                    throw new ServerException("Cannot stop the service.");
                }

                // Wait for the service to reach the desired status, or the timeout expired
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped, TimeSpan.FromMilliseconds(timeout));

            }
            catch (ServerException)
            {
                // do not wrap ServerException
                throw;
            }
            catch (Win32Exception e)
            {
                throw new ServerException("Fails to stop the server.", e);
            }
            catch (InvalidOperationException e)
            {
                throw new ServerException("Cannot stop the server.", e);
            }
            catch (System.ServiceProcess.TimeoutException e)
            {
                throw new ServerException("Cannot stop the server(timeout).", e);
            }
        }


        /// <summary>
        /// <p>
        /// Undeploy the application silently. Ignore the ServerException.
        /// </p>
        /// </summary>
        private void SilentUndeploy()
        {
            if (InitServer)
            {
                try
                {
                    Undeploy();
                }
                catch (ServerException)
                {
                    // ignore this exception
                }
            }
        }

        /// <summary>
        /// <p>
        /// Undeploy the application. This method removes all source files from deploy directory.
        /// </p>
        /// </summary>
        ///
        /// <exception cref="ServerException">if any error happens.</exception>
        private void Undeploy()
        {

            // creates a file set to delete
            FileSet deleteFileSet = new FileSet();
            deleteFileSet.Project = Project;
            deleteFileSet.BaseDirectory = DeploymentDirectory;

            // extract the included files from source file set
            FileSet.IncludesElement[] includes = new FileSet.IncludesElement[SourceFiles.Includes.Count];
            int index = 0;
            foreach (String pattern in SourceFiles.Includes)
            {
                includes[index] = new FileSet.IncludesElement();
                includes[index].Pattern = pattern;
                index ++;
            }

            // extract the excluded files from source file set
            FileSet.ExcludesElement[] excludes = new FileSet.ExcludesElement[SourceFiles.Excludes.Count];
            index = 0;
            foreach (String pattern in SourceFiles.Excludes)
            {
                excludes[index] = new FileSet.ExcludesElement();
                excludes[index].Pattern = pattern;
                index++;
            }

            deleteFileSet.SetIncludes = includes;
            deleteFileSet.SetExcludes = excludes;


            // create the DeleteTask and execute
            DeleteTask deleteTask = new DeleteTask();
            deleteTask.Project = Project;
            deleteTask.DeleteFileSet = deleteFileSet;


            try
            {
                deleteTask.Execute();
            }
            catch (ArgumentException e)
            {
                // ArgumentException may be thrown if the deployment directory is invalid
                throw new ServerException("Failed to remove the files.", e);
            }
            catch (BuildException e)
            {
                throw new ServerException("Failed to remove the files.", e);
            }

        }

    }

}

