/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NUnit.Framework;
using NAnt.Core;

namespace TopCoder.Test.Framework
{

    /// <summary>
    /// Unit test class for <c>ServerTaskException</c> class.
    /// </summary>
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    [TestFixture]
    public class ServerTaskExceptionTest
    {

        /// <summary>
        /// Represents the detail error message.
        /// </summary>
        private const string MESSAGE = "Error msg";

        /// <summary>
        /// Represents the original cause of the error.
        /// </summary>
        private Exception[] CAUSE = new Exception[] { new Exception("cause1"), new Exception("cause2") };

        /// <summary>
        /// Private <c>ServerTaskException</c> instance used for test.
        /// </summary>
        private ServerTaskException exception = null;

        /// <summary>
        /// Sets up the test environment. A valid <c>ServerTaskException</c> object is created.
        /// </summary>
        [SetUp]
        protected void SetUp()
        {
            exception = new ServerTaskException(MESSAGE, CAUSE);
        }

        /// <summary>
        /// Cleans up the test environment. The <c>ServerTaskException</c> object is disposed.
        /// </summary>
        [TearDown]
        protected void TearDown()
        {
            exception = null;
        }


        /// <summary>
        /// Tests inheritance of <c>ServerTaskException</c> class.
        /// The <c>ServerTaskException</c> class should be inherited from <c>BuildException</c> class.
        /// </summary>
        [Test]
        public void TestServerTaskExceptionAccuracyInherit()
        {
            Assert.IsTrue((object)new ServerTaskException(MESSAGE, CAUSE) is BuildException,
                "ServerTaskException should be inherited from BuildException.");
        }

        /// <summary>
        /// Tests accuracy of constructor of <c>ServerTaskException(string, Exception)</c>.
        /// The variable field should be set correctly.
        /// </summary>
        [Test]
        public void TestServerTaskExceptionStringExceptionAccuracy()
        {
            Assert.AreEqual(MESSAGE, exception.Message,
                "The detail message should be the same as that in argument.");
            Assert.AreEqual(CAUSE[0], exception.InnerException,
                "The cause should be the same as that in argument.");
            Assert.AreEqual(CAUSE, exception.InnerExceptions,
                "The cause should be the same as that in argument.");
        }

        /// <summary>
        /// Tests accuracy of constructor of <c>ServerTaskException(string, Exception)</c>.
        /// Null message string parameter should be accepted.
        /// The inner exception field should be set correctly.
        /// </summary>
        [Test]
        public void TestServerTaskExceptionStringExceptionAccuracyNullStringParam()
        {
            exception = new ServerTaskException(null, CAUSE);

            Assert.AreEqual(CAUSE[0], exception.InnerException,
                "The cause should be the same as that in argument.");
            Assert.AreEqual(CAUSE, exception.InnerExceptions,
                "The cause should be the same as that in argument.");
        }

        /// <summary>
        /// Tests accuracy of constructor of <c>ServerTaskException(string, Exception)</c>.
        /// Null value in array parameter should be accepted.
        /// </summary>
        [Test]
        public void TestServerTaskExceptionStringExceptionAccuracyNullInArrayParam()
        {
            exception = new ServerTaskException(null, new Exception[] { null });

            Assert.IsNotNull(exception, "Newly created object should not be null.");
        }

        /// <summary>
        /// Tests accuracy of constructor of <c>ServerTaskException(string, Exception)</c>.
        /// The message field should be set correctly.
        /// Null inner exception parameter should be accepted.
        /// </summary>
        [Test]
        public void TestServerTaskExceptionStringExceptionAccuracyNullExceptionParam()
        {
            exception = new ServerTaskException(MESSAGE, null);
            Exception inner = null;

            Assert.AreEqual(MESSAGE, exception.Message,
                "The detail message should be the same as that in argument.");
            Assert.AreEqual((new BuildException(MESSAGE, inner)).InnerException, exception.InnerException,
                "Inner exception should be set correctly.");
            Assert.IsNull(exception.InnerExceptions, "The InnerExceptions field should be set to null.");
        }

        /// <summary>
        /// Tests accuracy of constructor of <c>ServerTaskException(string, Exception)</c>.
        /// Null message string parameter should be accepted.
        /// Null inner exception parameter should be accepted.
        /// </summary>
        [Test]
        public void TestServerTaskExceptionStringExceptionAccuracyBothNullParam()
        {
            exception = new ServerTaskException(null, null);
            Exception inner = null;

            Assert.AreEqual((new BuildException(MESSAGE, inner)).InnerException, exception.InnerException,
                "Inner exception should be set correctly.");
            Assert.IsNull(exception.InnerExceptions, "The InnerExceptions field should be set to null.");
        }


        /// <summary>
        /// Tests accuracy of <c>InnerExceptions</c> getter property.
        /// Exception array returned should be correct.
        /// </summary>
        [Test]
        public void TestInnerExceptionsGetterAccuracy()
        {
            Assert.AreEqual(CAUSE, exception.InnerExceptions, "The cause returned should be correct.");
        }

        /// <summary>
        /// Tests accuracy of <c>InnerExceptions</c> getter property.
        /// Exception array returned should be correct.
        /// </summary>
        [Test]
        public void TestInnerExceptionsGetterAccuracyNullStringParam()
        {
            exception = new ServerTaskException(null, CAUSE);
            Assert.AreEqual(CAUSE, exception.InnerExceptions, "The cause returned should be correct.");
        }

        /// <summary>
        /// Tests accuracy of <c>InnerExceptions</c> getter property.
        /// Exception array returned should be correct.
        /// </summary>
        [Test]
        public void TestInnerExceptionsGetterAccuracyNullExceptionParam()
        {
            exception = new ServerTaskException(MESSAGE, null);
            Assert.IsNull(exception.InnerExceptions, "The cause returned should be null.");
        }

        /// <summary>
        /// Tests accuracy of <c>InnerExceptions</c> getter property.
        /// Exception array returned should be correct.
        /// </summary>
        [Test]
        public void TestInnerExceptionsGetterAccuracyBothNullParam()
        {
            exception = new ServerTaskException(null, null);
            Assert.IsNull(exception.InnerExceptions, "The cause returned should be null.");
        }
    }
}
