/*
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved.
 */
using System;
using NUnit.Framework;
using NAnt.Core.Types;
using NAnt.Core;
using System.Xml;
using TopCoder.Test.Framework.Servers;
using System.ServiceProcess;
using System.IO;

namespace TopCoder.Test.Framework
{
    /// <summary>
    /// Defines helper methods used in this component.
    /// This class is public, becuase it need be used in morn than one namespaces.
    /// </summary>
    ///
    /// <author>TCSDEVELOPER</author>
    /// <version>1.0</version>
    /// <copyright>Copyright (c) 2006, TopCoder, Inc. All rights reserved.</copyright>
    public sealed class TestFrameworkTestHelper
    {

        /// <summary>
        /// The base directory of test files.
        /// </summary>
        private const string BASE_DIR = "../../test_files/";

        /// <summary>
        /// The deploy directory
        /// </summary>
        private const string DEPLOY_DIR = BASE_DIR + "deploy/";

        /// <summary>
        /// The log file.
        /// </summary>
        private const string LOG_FILE = BASE_DIR + "log/HelloWorld.Tests.dll-results.txt";

        /// <summary>
        /// The IIS service name.
        /// </summary>
        private const string IIS_SERVICE_NAME = "w3svc";


        /// <summary>
        /// Constructor of <c>TestFrameworkTestHelper</c> class.
        /// </summary>
        ///
        /// <remarks>
        /// The private constructor prevents the instantiation.
        /// </remarks>
        private TestFrameworkTestHelper()
        {
        }


        /// <summary>
        /// Creates option with specified name and value.
        /// </summary>
        ///
        /// <param name="optionName">Name of option to create.</param>
        /// <param name="optionValue">Value of option to create.</param>
        /// <returns>Option object created.</returns>
        public static Option CreateOption(string optionName, string optionValue)
        {
            Option option = new Option();
            option.OptionName = optionName;
            option.Value = optionValue;

            return option;
        }

        /// <summary>
        /// Loads the server app test task from the given xml file.
        /// </summary>
        ///
        /// <param name="xmlFile">Name of the xml file.</param>
        /// <returns>The ServerAppTestTask instance.</returns>
        public static ServerAppTestTask LoadServerAppTestTask(String xmlFile)
        {
            // load the task element of given xml
            XmlDocument document = new XmlDocument();
            document.Load(BASE_DIR + xmlFile);
            XmlNode node = document.DocumentElement["serverapptest"];

            // load the task instance from the xml node
            ServerAppTestTask task = new ServerAppTestTask();
            task.Project = new Project(BASE_DIR + xmlFile, Level.Error);
            task.Initialize(node);
            return task;

        }

        /// <summary>
        /// Loads the server element from the given xml file.
        /// </summary>
        ///
        /// <param name="xmlFile">Name of the xml file.</param>
        /// <returns>The server element instance.</returns>
        public static ServerElement LoadServerElement(String xmlFile)
        {
            // load the server element of given xml
            XmlDocument document = new XmlDocument();
            document.Load(BASE_DIR + xmlFile);
            XmlNode serverNode = document.DocumentElement["server"];

            // load the ServerElement instance from the xml node
            ServerElement serverElement = new ServerElement();
            serverElement.Project = new Project(BASE_DIR + xmlFile, Level.Error);
            serverElement.Initialize(serverNode);
            return serverElement;

        }

        /// <summary>
        /// Loads the server instance from the given xml file.
        /// </summary>
        ///
        /// <param name="xmlFile">Name of the xml file.</param>
        /// <returns>The server instance.</returns>
        public static Server LoadServer(String xmlFile)
        {
            return LoadServerElement(xmlFile).GetServerInstance();
        }

        /// <summary>
        /// Starts the IIS service.
        /// </summary>
        public static void StartService()
        {
            ServiceController serviceController = new ServiceController(IIS_SERVICE_NAME);

            // Start the service
            if (serviceController.Status == ServiceControllerStatus.Paused)
            {
                serviceController.Continue();
                serviceController.WaitForStatus(ServiceControllerStatus.Running);
            }
            else if (serviceController.Status == ServiceControllerStatus.Stopped)
            {
                serviceController.Start();
                serviceController.WaitForStatus(ServiceControllerStatus.Running);
            }


        }

        /// <summary>
        /// Stop the IIS service.
        /// </summary>
        public static void StopService()
        {
            ServiceController serviceController = new ServiceController(IIS_SERVICE_NAME);
            if (serviceController.Status == ServiceControllerStatus.Running && serviceController.CanStop)
            {
                serviceController.Stop();
                serviceController.WaitForStatus(ServiceControllerStatus.Stopped);
            }

        }

        /// <summary>
        /// Gets whether IIS service is started.
        /// </summary>
        /// <returns>whether IIS service is started</returns>
        public static bool IsServiceStarted()
        {
            return new ServiceController(IIS_SERVICE_NAME).Status == ServiceControllerStatus.Running;
        }

        /// <summary>
        /// Gets whether given file is deployed.
        /// </summary>
        /// <param name="name">Name of the file.</param>
        /// <returns>whether given file is deployed.</returns>
        public static bool FileDeployed(string name)
        {
            return new FileInfo(DEPLOY_DIR + name).Exists;
        }

        /// <summary>
        /// Deletes the given file.
        /// </summary>
        /// <param name="name">Name of the file.</param>
        public static void DeleteFile(string name)
        {
            if (FileDeployed(name))
            {
                new FileInfo(BASE_DIR + name).Delete();
            }
        }

        /// <summary>
        /// Checks whether the helloworld tests passed.
        /// </summary>
        /// <returns>whether the helloworld tests passed</returns>
        public static bool CheckLog()
        {
            using (TextReader reader = new StreamReader(new FileStream(LOG_FILE, FileMode.Open)))
            {
                String result = reader.ReadLine();
                return result.StartsWith("Tests run: 1, Failures: 0, Not run: 0");
            }
        }

    }
}
